==> xl-react-components-0.16.62.tgz: package/app/xl-react-dip/xl-react-dip-list/xl-react-dip-list-component.js <==
import PropTypes from 'prop-types';
import R from 'ramda';
import React, {PureComponent} from 'react';
import classNames from 'classnames';
import RU from '../../utils/ramda-utils';
import {localeMessages} from '../../services/locale-service';
import SortableList from '../sortable-list/sortable-list';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {validationMessageClasses} from '../../utils/css-utils';
import {XlReactWidgetAutocomplete} from '../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete'; // eslint-disable-line max-len
import {XlReactWidgetString} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';

import './xl-react-dip-list.less';

export class XlReactDipListComponent extends PureComponent {
    static propTypes = {
        autocompleteMetadata: PropTypes.object,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        handleAddItem: PropTypes.func,
        handlers: PropTypes.object,
        handleSortableListChange: PropTypes.func,
        handleSuggestionSelection: PropTypes.func,
        handleUserInput: PropTypes.func,
        label: PropTypes.string,
        message: xlMessageTypeProp,
        model: PropTypes.array,
        name: PropTypes.string,
        objLabel: PropTypes.string,
        onMessageChange: PropTypes.func,
        onRemoveSortableListItem: PropTypes.func,
        onValidityChange: PropTypes.func,
        registerChildApi: PropTypes.func,
        required: PropTypes.bool,
        typed: PropTypes.string,
        valid: PropTypes.bool,
        validators: PropTypes.array
    };

    inputFieldMetadata = {
        default: '',
        name: this.props.name,
        onUserInput: this.props.handleUserInput,
        validators: this.props.validators
    };

    renderComponent() {
        const {
            autocompleteMetadata,
            disabled,
            displayMode,
            handleSuggestionSelection,
            handleUserInput,
            handlers,
            objLabel,
            onMessageChange,
            onValidityChange,
            registerChildApi,
            typed
        } = this.props;

        return handlers ?
            <XlReactWidgetAutocomplete
                disabled={disabled}
                displayMode={displayMode}
                handlers={handlers}
                metadata={autocompleteMetadata}
                model={typed}
                objLabel={objLabel}
                onMessageChange={onMessageChange}
                onModelChange={handleUserInput}
                onSuggestionSelected={handleSuggestionSelection}
                onValidityChange={onValidityChange}
                register={registerChildApi}
            /> :
            <XlReactWidgetString
                disabled={disabled}
                metadata={this.inputFieldMetadata}
                model={typed}
                register={registerChildApi}
            />;
    }

    render() {
        const {
            description,
            disabled,
            displayMode,
            message,
            handleAddItem,
            handleSortableListChange,
            label,
            model,
            onRemoveSortableListItem,
            required,
            typed,
            valid
        } = this.props;
        const isReadOnly = displayMode === 'visual';
        const indexedItems = RU.indexedArray(model);

        const scrollablePanel = () => <div className="xl-scrollable-panel">
            <div>
                <SortableList
                    canSort={!disabled}
                    disabled={disabled}
                    indexedItems={indexedItems}
                    onChange={handleSortableListChange}
                    remove={onRemoveSortableListItem}
                />
            </div>
        </div>;

        const addButton = () => isReadOnly ? null : <button
            className="xl-react-button xl-primary xl-btn-add"
            disabled={!typed || disabled}
            onClick={handleAddItem}
            type="button">
            {localeMessages.addButton}
        </button>;

        const addItemComponent = () => {
            const showAddItem = () => R.isEmpty(model) && isReadOnly || !isReadOnly;

            const dipInputClass = classNames('dip-input', {visual: isReadOnly});

            return showAddItem() ?
                <div>
                    {addButton()}
                    <div className={dipInputClass}>
                        {this.renderComponent()}
                    </div>
                </div> : null;
        };
        /* eslint-disable sort-keys */
        const controlProps = {
            controlClass: 'xl-react-dip-list',
            customControlClasses: validationMessageClasses(valid, message),
            description,
            disabled,
            label,
            message,
            required
        };
        /* eslint-enable sort-keys */

        return <XlReactFormItemControl {...controlProps}>
            {addItemComponent()}
            {scrollablePanel()}
        </XlReactFormItemControl>;
    }
}

==> xl-react-components-0.18.103.tgz: package/app/xl-react-dip/xl-react-dip-list/xl-react-dip-list-component.js <==
import PropTypes from 'prop-types';
import R from 'ramda';
import React, {PureComponent} from 'react';
import classNames from 'classnames';
import {localeMessages} from '../../services/locale-service';
import {
    prepareViewModel,
    revalidateItems
} from '../xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service';
import SortableList from '../sortable-list/sortable-list';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {validationMessageClasses} from '../../utils/css-utils';
import {XlReactWidgetAutocomplete}
    from '../../xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete';
import {XlReactWidgetString} from '../../xl-react-widget/xl-react-widget-string/xl-react-widget-string';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';

import './xl-react-dip-list.less';

export class XlReactDipListComponent extends PureComponent {
    static propTypes = {
        autocompleteMetadata: PropTypes.object,
        description: PropTypes.string,
        disabled: PropTypes.bool,
        displayMode: PropTypes.string,
        handleAddItem: PropTypes.func,
        handlers: PropTypes.object,
        handleSortableListChange: PropTypes.func,
        handleSuggestionSelection: PropTypes.func,
        handleUserInput: PropTypes.func,
        label: PropTypes.string,
        message: xlMessageTypeProp,
        model: PropTypes.array,
        name: PropTypes.string,
        objLabel: PropTypes.string,
        onMessageChange: PropTypes.func,
        onRemoveSortableListItem: PropTypes.func,
        onValidityChange: PropTypes.func,
        registerChildApi: PropTypes.func,
        required: PropTypes.bool,
        typed: PropTypes.string,
        valid: PropTypes.bool,
        validators: PropTypes.array
    };

    constructor(props) {
        super(props);
        this.state = {visibleItems: prepareViewModel(props.model, props.objLabel, true)};
    }

    componentDidMount() {
        revalidateItems(this, this.props.model);
    }

    componentWillReceiveProps(nextProps) {
        if (!R.equals(nextProps.model, this.props.model)) {
            revalidateItems(this, nextProps.model);
        }
    }

    inputFieldMetadata = {
        default: '',
        name: this.props.name,
        onUserInput: this.props.handleUserInput,
        validators: this.props.validators
    };

    renderComponent() {
        const {
            autocompleteMetadata,
            disabled,
            displayMode,
            handleSuggestionSelection,
            handleUserInput,
            handlers,
            objLabel,
            onMessageChange,
            onValidityChange,
            registerChildApi,
            typed
        } = this.props;

        return handlers ?
            <XlReactWidgetAutocomplete
                disabled={disabled}
                displayMode={displayMode}
                handlers={handlers}
                metadata={autocompleteMetadata}
                model={typed}
                objLabel={objLabel}
                onMessageChange={onMessageChange}
                onModelChange={handleUserInput}
                onSuggestionSelected={handleSuggestionSelection}
                onValidityChange={onValidityChange}
                register={registerChildApi}
            /> :
            <XlReactWidgetString
                disabled={disabled}
                metadata={this.inputFieldMetadata}
                model={typed}
                register={registerChildApi}
            />;
    }

    render() {
        const {
            description,
            disabled,
            displayMode,
            message,
            handleAddItem,
            handleSortableListChange,
            label,
            model,
            onRemoveSortableListItem,
            required,
            typed,
            valid
        } = this.props;
        const {visibleItems} = this.state;
        const isReadOnly = displayMode === 'visual';

        const scrollablePanel = () => <div className="xl-scrollable-panel">
            <div>
                <SortableList
                    canSort={!disabled}
                    disabled={disabled}
                    indexedItems={visibleItems}
                    onChange={handleSortableListChange}
                    remove={onRemoveSortableListItem}
                />
            </div>
        </div>;

        const addButton = () => isReadOnly ? null : <button
            className="xl-react-button xl-primary xl-btn-add"
            disabled={!typed || disabled}
            onClick={handleAddItem}
            type="button">
            {localeMessages.addButton}
        </button>;

        const addItemComponent = () => {
            const showAddItem = () => R.isEmpty(model) && isReadOnly || !isReadOnly;

            const dipInputClass = classNames('dip-input', {visual: isReadOnly});

            return showAddItem() ?
                <div>
                    {addButton()}
                    <div className={dipInputClass}>
                        {this.renderComponent()}
                    </div>
                </div> : null;
        };
        /* eslint-disable sort-keys */
        const controlProps = {
            controlClass: 'xl-react-dip-list',
            customControlClasses: validationMessageClasses(valid, message),
            description,
            disabled,
            label,
            message,
            required
        };
        /* eslint-enable sort-keys */

        return <XlReactFormItemControl {...controlProps}>
            {addItemComponent()}
            {scrollablePanel()}
        </XlReactFormItemControl>;
    }
}
